<?php
/**
 * Composite Phrase renderer
 *
 * Copyright © 2013-2017 Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */
namespace FiloBlu\Label\Model\Translate;

use Magento\Framework\Phrase\RendererInterface;

class Composite extends \Magento\Framework\Phrase\Renderer\Composite implements RendererInterface
{

    protected $scopeConfig;
    protected $appState;

    /**
     * @param \Magento\Framework\Phrase\RendererInterface[] $renderers
     * @throws \InvalidArgumentException
     */
    public function __construct(
        array $renderers,
        \Magento\Framework\App\State $appState,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Locale\Resolver $localeResolver,
        \Magento\Translation\Model\ResourceModel\StringUtilsFactory $resource
        )
    {

        $this->appState = $appState;
        $this->scopeConfig = $scopeConfig;
        $this->_storeManager = $storeManager;
        $this->_localeResolver = $localeResolver;
        $this->_resourceFactory = $resource;

        foreach ($renderers as $renderer) {
            if (!$renderer instanceof RendererInterface) {
                throw new \InvalidArgumentException(
                    sprintf('Instance of the phrase renderer is expected, got %s instead.', get_class($renderer))
                );
            }
        }
        $this->_renderers = $renderers;
    }


    /**
     * Render source text
     *
     * @param [] $source
     * @param [] $arguments
     * @return string
     */
    public function render(array $source, array $arguments = [])
    {
        $result = $source;
        foreach ($this->_renderers as $render) {
            $result[] = $render->render($result, $arguments);
        }

        $label_result = end($result);

        if ($this->appState->getAreaCode() != \Magento\Framework\App\Area::AREA_ADMINHTML && $this->scopeConfig->getValue('dev/registration_inline/active') && strpos($result[0], "%") === false) {

            $locale = $this->_localeResolver->getLocale();
            $validStoreId = $this->_storeManager->getStore()->getId();
            $this->saveTranslate($result[0],$label_result,$locale,$validStoreId);

        }


        return $label_result;
    }

    /**
     * Save translation
     *
     * @param String $string
     * @param String $translate
     * @param String $locale
     * @param int|null $storeId
     * @return $this
     */
    public function saveTranslate($string, $translate, $locale = null, $storeId = null)
    {
        $string = htmlspecialchars_decode($string);
        /** @var $resource \Magento\Translation\Model\ResourceModel\StringUtils */
        $resource = $this->_resourceFactory->create();
        $connection = $resource->getConnection();
        $table = $resource->getMainTable();
        $translate = htmlspecialchars($translate, ENT_QUOTES);

        if ($locale === null) {
            $locale = $this->_localeResolver->getLocale();
        }

        if ($storeId === null) {
            $storeId = $this->getStoreId();
        }

        $select = $connection->select()->from(
            $table,
            ['key_id', 'translate']
            )->where(
                'store_id = :store_id'
            )->where(
                'locale = :locale'
            )->where(
                'string = :string'
            )->where(
                'crc_string = :crc_string'
            );
        $bind = [
            'store_id' => $storeId,
            'locale' => $locale,
            'string' => $string,
            'crc_string' => crc32($string),
        ];

        if ($row = $connection->fetchRow($select, $bind)) {
//            if ($row['translate'] != $translate) {
//                $connection->update($table, ['translate' => $translate], ['key_id=?' => $row['key_id']]);
//            }
        } else {
            $connection->insert(
                $table,
                [
                    'store_id' => $storeId,
                    'locale' => $locale,
                    'string' => $string,
                    'translate' => $translate,
                    'crc_string' => crc32($string)
                ]
            );
        }

        return $this;
    }
}
